package com.macro.mall.tiny.modules.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.macro.mall.tiny.common.exception.Asserts;
import com.macro.mall.tiny.common.util.MyFileUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.domain.AdminUserDetails;
import com.macro.mall.tiny.modules.business.enums.DirectoryEnum;
import com.macro.mall.tiny.modules.business.mapper.BLineMapper;
import com.macro.mall.tiny.modules.business.mapper.BTowerMapper;
import com.macro.mall.tiny.modules.business.model.*;
import com.macro.mall.tiny.modules.business.service.BDirectoryCacheService;
import com.macro.mall.tiny.modules.business.service.BLineCacheService;
import com.macro.mall.tiny.modules.business.service.BTowerService;
import com.macro.mall.tiny.modules.business.service.FileService;
import com.macro.mall.tiny.security.util.AuthUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

import static com.macro.mall.tiny.common.util.UploadUtils.*;


/**
 * 文件上传服务
 */
@Service
@Slf4j
@EnableAsync
public class FileServiceImpl implements FileService {

    @Autowired
    private AsyncHandler asyncHandler;

    @Autowired
    private BDirectoryCacheService directoryCacheService;

    @Autowired
    private BTowerService towerService;

    @Autowired
    private BTowerMapper towerMapper;

    @Autowired
    private BLineMapper lineMapper;

    @Autowired
    private BLineCacheService lineCacheService;

    /**
     * 上传文件
     *
     * @param file
     */
    public void upload(MultipartFile file) throws IOException {
        String path = UploadConfig.path + file.getOriginalFilename();
        MyFileUtils.write(path, file.getInputStream());
    }

    /**
     * 分块上传文件
     *
     * @param size
     * @param chunks
     * @param chunk
     * @param file
     * @throws IOException
     */
    public void uploadWithBlock(String name,
                                String lineName,
                                Long provinceId,
                                Long size,
                                Integer chunks,
                                Integer chunk,
                                MultipartFile file,
                                AdminUserDetails userDetails,
                                String charset) throws IOException {
        putChunkMap(name, chunks);
        String target = UploadConfig.tempPath + name;
        MyFileUtils.writeWithBlock(target, size, file.getInputStream(), file.getSize(), chunks, chunk);
        addChunk(name, chunk);
        if (isUploaded(name)) {
            removeKey(name);
            asyncHandler.handleDocument(provinceId, target, lineName, userDetails, charset);
        }
    }

    @Override
    public BDirectory getRootDirectory(AdminUserDetails userDetails) {
        BDirectory directory = directoryCacheService.getDirectory(userDetails);
        if (directory != null) {
            return directory;
        }
        directory = MyFileUtils.getRootDirectory(userDetails);
        if (directory != null) {
            directoryCacheService.setDirectory(userDetails, directory);
        } else {
            directoryCacheService.setDirectory(userDetails, new BDirectory());
        }
        return directory;
    }

    @Override
    public List<BFileDetail> getFileDetailList(String path, AdminUserDetails userDetails) {
        return MyFileUtils.getFileDetailList(path, userDetails);
    }

    @Override
    public List<BFile> getFileList(String path, AdminUserDetails userDetails) {
        return MyFileUtils.getSonFiles(path, userDetails);
    }

    @Override
    public boolean deleteFile(String path, AdminUserDetails userDetails) throws IOException {
        boolean delete = AuthUtil.checkAccess(userDetails, AuthUtil.DELETE);
        Path filePath = Paths.get(path);
        if (!delete) {
            Asserts.fail("您没有删除权限");
        }
        if (Files.exists(filePath)) {
            Files.delete(filePath);
        } else {
            Asserts.fail("该文件不存在");
        }
        int count = filePath.getNameCount();
        if (count < 5) {
            return true;
        }
        String lineNameStr = filePath.getName(3).toString();
        String dirType = filePath.getName(4).toString();

        if (!dirType.equals(DirectoryEnum.TOWER_PIC.getName()) && !dirType.equals(DirectoryEnum.EARTH_WIRE_HARD_PIC.getName()) && !dirType.equals(DirectoryEnum.TEAM_CHECK_PIC.getName())) {
            return true;
        }

        QueryWrapper<BLine> lineQueryWrapper = new QueryWrapper<>();
        lineQueryWrapper.lambda().eq(BLine::getName, lineNameStr);
        BLine line = lineMapper.selectOne(lineQueryWrapper);

        QueryWrapper<BTower> towerQueryWrapper = new QueryWrapper<>();
        towerQueryWrapper.lambda().eq(BTower::getLineId, line.getId());
        List<BTower> towerList = towerMapper.selectList(towerQueryWrapper);
        towerService.setHasFile(towerList, userDetails);

        // 后续清楚缓存处理
        lineCacheService.delLineList();
        directoryCacheService.delDirectory();
        return true;
    }

}
